###############################################################################
#
# Worksheet - A class for writing Excel Worksheets.
#
# Copyright 2013-2019, John McNamara, jmcnamara@cpan.org
#
# https://github.com/jmcnamara/XlsxWriter/blob/master/xlsxwriter/utility.py
#

from warnings import warn


def xl_col_to_name(col, col_abs=False):
    """
    Convert a zero indexed column cell reference to a string.

    Args:
       col:     The cell column. Int.
       col_abs: Optional flag to make the column absolute. Bool.

    Returns:
        Column style string.

    """
    col_num = col
    if col_num < 0:
        warn("Col number %d must be >= 0" % col_num)
        return None

    col_num += 1  # Change to 1-index.
    col_str = ''
    col_abs = '$' if col_abs else ''

    while col_num:
        # Set remainder from 1 .. 26
        remainder = col_num % 26

        if remainder == 0:
            remainder = 26

        # Convert the remainder to a character.
        col_letter = chr(ord('A') + remainder - 1)

        # Accumulate the column letters, right to left.
        col_str = col_letter + col_str

        # Get the next order of magnitude.
        col_num = int((col_num - 1) / 26)

    return col_abs + col_str



import os
import sys
import random


def django_setup(project_name=None, site_path=None):
    """
    设置 Django 运行环境
    
    https://stackoverflow.com/questions/8047204/django-script-to-access-model-objects-without-using-manage-py-shell

    from ilds.django.util import django_setup
    django_setup(r'mysite', site_path=None)
    """

    if site_path is not None:
        sys.path.insert(0, site_path)

    if project_name is None:
        project_name = os.path.split(os.path.dirname(__file__))[-1]
    print('项目：', project_name)
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "{}.settings".format(project_name))
    try:
        import django
        django.setup()
    except ModuleNotFoundError as e:
        raise ModuleNotFoundError(
            f"\n\n{e}\n\n注：如果提示 ModuleNotFoundError: No module named 'django'，请安装 django: pip install django")


def random_key():
    """
    生成 Django 使用的 SECRET_KEY

    from ilds.django.util import random_key
    print(random_key())
    """

    return ''.join(
        [random.SystemRandom().choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])



import os
import sys

from setuptools import setup, find_packages

from djlds.versions import version

"""
pip install -U spider-utils
pip --no-cache-dir install -U spider-utils

# 检查错误
# twine check dist/*

echo 使用 twine 上传到官方的pip服务器:
echo 在系统添加 TWINE_USERNAME 和 TWINE_PASSWORD 变量，不用输入用户名和密码
echo 例如 TWINE_USERNAME=__token__ TWINE_PASSWORD=pypi-AgEIcHlwaS5vcmcCJD...

rmdir /S/Q build
rmdir /S/Q dist
python setup.py sdist bdist_wheel
echo 上传到PyPI:
twine upload dist/*

"""

# twine upload dist/* 使用 twine 上传
# 添加上传到 PyPI 的命令
# 设置 TWINE_USERNAME=lds 和 TWINE_PASSWORD 变量，但不建议设置到系统里面
# 勾选：Emulate terminal in output console(在输出控制台中模拟终端)
if sys.argv[-1] == 'up':
    # os.system('rm -rf dist')
    # os.system('rm -rf build')
    os.system('rmdir /S/Q build')
    os.system('rmdir /S/Q dist')
    os.system('python setup.py sdist bdist_wheel')
    os.system('twine check dist/*')
    os.system('twine upload dist/*')
    sys.exit()
elif sys.argv[-1] == 'dev':
    os.system('pip install wheel')
    os.system('pip install twine')
    sys.exit()

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    # 名称
    name="djlds",
    # 版本
    version=version,
    # version=".".join(map(str, __import__('html2text').__version__)),
    # 关键字列表
    keywords=("django", "utils"),
    # 简单描述
    description="常用 Django 功能集合，为了多平台，多电脑调用方便!",
    # 详细描述
    long_description=long_description,
    long_description_content_type="text/markdown",
    # 授权信息
    license="GNU GPL 3",

    # 官网地址
    url="https://github.com/ldsxp/django-utils-lds",
    # 程序的下载地址
    download_url="https://pypi.org/project/django-utils-lds",
    # 作者
    author="lds",
    # 作者的邮箱地址
    author_email="85176878@qq.com",

    # 需要处理的包目录（包含__init__.py的文件夹）
    packages=find_packages('.', exclude=['tests', 'tests.*']),
    # 软件平台列表
    platforms="any",
    # 所属分类列表
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
    ],
    # 需要安装的依赖包
    install_requires=[
        'ilds',
        'colorama',
        'django-admin-generator>=2.6.0',
        'Django',
    ],
    include_package_data=True,
    extras_require={'dev': ['wheel', 'twine', ]},
    python_requires='>=3.6',

    zip_safe=False
)



# ---------------------------------------
#   程序：user.py
#   版本：0.2
#   作者：lds
#   日期：2022-01-21
#   语言：Python 3.X
#   说明：django 用户相关的函数集合
# ---------------------------------------

# from django.contrib.auth.models import User
from django.contrib.auth import get_user_model
from django.contrib.auth.hashers import make_password  # , check_password

User = get_user_model()


def create_user(username, email, password, is_superuser=False, is_staff=True, is_active=True):
    """
    创建用户

    例子：
    from ilds.django.user import create_user
    create_user(username='admin', email='admin@qq.com', password='123456', is_superuser=False, is_staff=True)

    :param username: 用户名
    :param email: 邮箱
    :param password: 密码，支持直接设置秘钥，秘钥前缀是 pbkdf2_
    :param is_superuser: 是否是超级用户
    :param is_staff: 用户是否可以登录到管理站点
    :param is_active: 用户是否被认为是活跃的。可以用来代替删除帐号
    :return: 创建成功的用户
    """
    if not password.startswith('pbkdf2_'):
        password = make_password(password)

    # 添加一个超级用户数据
    add_user = User.objects.create(**{
        'password': password,
        'last_login': None,
        'is_superuser': is_superuser,
        'username': username,
        'email': email,
        'is_staff': is_staff,
        'is_active': is_active
    })

    try:
        from allauth.account.models import EmailAddress  # django-allauth
        EmailAddress.objects.create(**{
            'user': add_user,
            'email': email,
            'verified': True,
            'primary': True})
    except Exception as e:
        print(e)

    return add_user


def add_superuser(username, email, password):
    """
    添加超级管理员用户

    例子：
    from ilds.django.user import add_superuser
    add_superuser(username='admin', email='admin@qq.com', password='123456')

    :param username: 用户名
    :param email: 邮箱
    :param password: 密码，支持直接设置秘钥，秘钥前缀是 pbkdf2_
    :return: 创建成功的用户
    """
    return create_user(username, email, password, is_superuser=True, is_staff=True, is_active=True)
